package org.jaggregator;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Properties;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections.Predicate;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.Category;
import org.apache.log4j.PropertyConfigurator;
import org.jaggregator.config.Configuration;
import org.jaggregator.config.ConfigurationParser;
import org.jaggregator.config.SubscriptionDecl;
import org.jaggregator.config.SubscriptionListDecl;
import org.jaggregator.generator.SiteGenerationException;
import org.jaggregator.generator.SiteGenerationTrigger;
import org.jaggregator.generator.SiteGenerator;
import org.jaggregator.rss.RssException;
import org.jaggregator.rss.Subscription;
import org.jaggregator.rss.util.SubscriptionRegistry;
import org.jaggregator.util.FileUtils;
import org.quartz.SchedulerException;
import org.xml.sax.SAXException;

/**
 * Entry point to org.jaggregator.
 *
 * @author <a href="mailto:jsz@jaggregator.org">Janek Schwarz</a>
 * @version $Revision: 1.37 $
 */
public class Main
  implements Runnable
{
private static final String LOG4J_CONFIGURATION_RESOURCE =
  "resources/org/jaggregator/log4j.ini";
private static final String VERSION_RESOURCE =
  "resources/org/jaggregator/version.ini";
private static final String CATEGORY_NAME =
  "org.jaggregator";
public static final String VERSION_STRING;

private Category log;
private String[] args;
private SubscriptionRegistry registry;
private Configuration configuration;

private boolean forceSiteGeneration = false;

static
{
  ClassLoader threadClassLoader =
    Thread.currentThread().getContextClassLoader();
  URL log4jConfigurationUrl =
    threadClassLoader.getResource(LOG4J_CONFIGURATION_RESOURCE);

  if(log4jConfigurationUrl == null)
    {
      BasicConfigurator.configure();
      Category.getInstance(CATEGORY_NAME).warn("Can't find " +
                    LOG4J_CONFIGURATION_RESOURCE +
                    ". Use defaults.");
    }
  else
    {
      PropertyConfigurator.configure(log4jConfigurationUrl);
    }

  InputStream versionPropertiesIstream =
    threadClassLoader.getResourceAsStream(VERSION_RESOURCE);

  String versionString = "";
  if(versionPropertiesIstream == null)
    {
      Category.getInstance(CATEGORY_NAME).warn("can't resolve resource: " +
                                               VERSION_RESOURCE);
    }
  else
    {
      try
        {
          Properties versionProperties = new Properties();
          versionProperties.load(versionPropertiesIstream);

          versionString =
            versionProperties.getProperty("jaggregator.version", "");
        }
      catch(IOException ex)
        {
          Category.getInstance(CATEGORY_NAME).warn(
            "can't resolve resource: " +
            VERSION_RESOURCE);
          Category.getInstance(CATEGORY_NAME).debug(
            ExceptionUtils.getStackTrace(ex));

          versionString = "";
        }
    }

  VERSION_STRING = versionString;
}

public Main(String[] args)
{
  this.args = args;

  for(int i = 0; i < args.length; i++)
    {
      if(args[i].equals("-fg"))
        forceSiteGeneration = true;
    }
}

public void run()
{
  try
    {
      log = Category.getInstance(CATEGORY_NAME);

      log.debug(new Date());

      configuration = openConfiguration();
      log.debug("opened and read configuration file");

      FileUtils.setupDirectory(configuration.getSubscriptionsDirectory());
      FileUtils.setupDirectory(configuration.getOutputDirectory());

      registry = openSubscriptionRegistry();
      log.debug("opened subscription registry");

      configureSubscriptionRegistry();
      log.debug("configured subscription registry");

      log.debug("already subscribed to: " + registry.getSubscriptionUris());
      synchronizeSubscriptionRegistry(configuration);

      SiteGenerator generator =
        new SiteGenerator(registry.getSubscriptionsMap(),
                          configuration.getSkinDirectory(),
                          configuration.getOutputDirectory(),
                          configuration.getSubscriptionLists(),
                          configuration.getTemplates());
      generator.initialize();
      log.debug("created and initialized site generator");

      SiteGenerationTrigger siteGenTrigger =
        new SiteGenerationTrigger(generator);
      registry.addChannelRetrievalListener(siteGenTrigger);
      registry.scheduleRetrieval();
      log.debug("scheduled retrieval of channel data");

      log.info("JAggregator is up and running. Exit using CTRL-C");

      if(forceSiteGeneration)
        {
          log.debug("user forced site generation");
          generator.generate(registry.getSubscriptionsMap().keySet());
        }

      while(true)
        {
          Date nextUpdateDate = registry.getNextUpdateTime();
          log.debug("next update scheduled for: " + nextUpdateDate);
          log.debug("go to sleep");

          Thread.currentThread().sleep(60L * 1000L);
        }
    }
  catch(MalformedURLException ex)
    {
      log.error(ex.getMessage());
      log.debug(ExceptionUtils.getStackTrace(ex));
    }
  catch(RssException ex)
    {
      log.error(ex.getMessage());
      log.debug(ExceptionUtils.getStackTrace(ex));
    }
  catch(IOException ex)
    {
      log.error(ex.getMessage());
      log.debug(ExceptionUtils.getStackTrace(ex));
    }
  catch(SiteGenerationException ex)
    {
      log.error(ex.getMessage());
      log.debug(ExceptionUtils.getStackTrace(ex));
    }
  catch(InterruptedException ex)
    {
      log.error(ex.getMessage());
      log.debug(ExceptionUtils.getStackTrace(ex));
      Thread.currentThread().interrupt();
    }
  catch(SAXException ex)
    {
      log.error("Error when reading configuration file: " + ex.getMessage());
      log.debug(ExceptionUtils.getStackTrace(ex));
    }
  catch(SchedulerException ex)
    {
      log.error(ex.getMessage());
      log.debug(ExceptionUtils.getStackTrace(ex));
    }
}

private SubscriptionRegistry openSubscriptionRegistry()
{
  SubscriptionRegistry registry =
    SubscriptionRegistry.getInstance(
      configuration.getSubscriptionsDirectory());
  boolean errorsOccured = registry.initialize();

  if(errorsOccured)
    {
      Iterator iter = registry.getInitializationExceptionList().iterator();
      while(iter.hasNext())
        {
          SubscriptionRegistry.DirectoryNameExceptionTupel tupel =
            (SubscriptionRegistry.DirectoryNameExceptionTupel) iter.next();

          log.error("directory: " + tupel.getDirectoryName() +
                    "\ncaught exception: " +
                    ExceptionUtils.getStackTrace(tupel.getException()));
        }
    }

  return registry;
}

private void configureSubscriptionRegistry()
  throws IOException
{
  registry.setDefaultRefreshRate(configuration.getRefreshRate());

  if(configuration.isProxySet())
    {
      registry.setProxy(configuration.getProxyHost(),
                        configuration.getProxyPort());
    }

  if(configuration.hasWebLogUrl())
    registry.addQueryStringPart("userWebLog",
                                configuration.getWebLogUrl());

  String userAgentString = "JAggregator";

  if(VERSION_STRING.length() > 0)
    userAgentString = userAgentString + "/" + VERSION_STRING;

  registry.addHttpHeader("User-Agent", userAgentString);
}

private Configuration openConfiguration()
  throws IOException, SAXException
{
  ConfigurationParser parser =
    new ConfigurationParser(new File("jaggregator.xml"));
  Configuration configuration = parser.parse();

  return configuration;
}

private void synchronizeSubscriptionRegistry(Configuration configuration)
  throws RssException, IOException
{
  Collection configurationSubscriptionLists =
    configuration.getSubscriptionLists();
  Iterator subscriptionListIter = configurationSubscriptionLists.iterator();

  while(subscriptionListIter.hasNext())
    {
      SubscriptionListDecl list =
        (SubscriptionListDecl) subscriptionListIter.next();
      Iterator subscriptionIter = list.values().iterator();

      while(subscriptionIter.hasNext())
        {
          SubscriptionDecl subscriptionDecl =
            (SubscriptionDecl) subscriptionIter.next();
          URL channelUri = subscriptionDecl.getChannelUri();

          if(registry.containsSubscription(channelUri) == false)
            {
              registry.subscribe(channelUri, true);

              log.debug("subscribed to channel: " + channelUri);
            }
        }
    }

  Collection toBeUnsubscribedCollection = new LinkedList();
  Collection subscriptionCollection = registry.getSubscriptions();
  Iterator subscriptionIter = subscriptionCollection.iterator();
  while(subscriptionIter.hasNext())
    {
      final URL channelUri = ((Subscription) subscriptionIter.next()).getUrl();

      boolean found = false;
      Iterator iter = configurationSubscriptionLists.iterator();
      while(iter.hasNext())
        {
          SubscriptionListDecl subscriptionListDecl =
            (SubscriptionListDecl) iter.next();

          SubscriptionDecl subscriptionDecl =
            (SubscriptionDecl) CollectionUtils.find(
              subscriptionListDecl.values(),
              new Predicate()
              {
              public boolean evaluate(Object input)
              {
                SubscriptionDecl sc = (SubscriptionDecl) input;

                return channelUri.equals(sc.getChannelUri());
              }
              }
              );

          if(subscriptionDecl != null)
            {
              found = true;
              break;
            }
        }

      if(found == false)
        toBeUnsubscribedCollection.add(channelUri);
    }

  if(toBeUnsubscribedCollection.size() > 0)
    {
      log.debug("unsubscribe from: " + toBeUnsubscribedCollection);
      registry.unsubscribe(toBeUnsubscribedCollection);
    }
}

public static void main(String[] args)
{
  Main main = new Main(args);

  main.run();

  System.exit(0);
}
}
