package org.jaggregator.generator;

import java.io.File;
import java.io.IOException;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.apache.log4j.Category;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.exception.ParseErrorException;
import org.apache.velocity.exception.ResourceNotFoundException;
import org.jaggregator.config.SubscriptionListDecl;
import org.jaggregator.config.TemplateDecl;
import org.jaggregator.generator.Invoker;
import org.jaggregator.generator.SiteGenerationException;


/**
 * Abstract class that provides a couple of methods and instance
 * variables to ease Invoker development.
 *
 * @author <a href="mailto:jsz@jaggregator.de">Janek Schwarz</a>
 * @version $Revision: 1.3 $
 */
public abstract class AbstractInvoker 
 implements Invoker
{
private Category log = Category.getInstance(AbstractInvoker.class);

protected File outputDirectory;
protected HrefGenerator hrefGenerator;

protected AbstractInvoker() 
{}

public void setOutputDirectory(File outputDirectory)
{
  this.outputDirectory = outputDirectory;

  if(hrefGenerator == null)
    hrefGenerator = new HrefGenerator(outputDirectory);
  else
    hrefGenerator.setOutputDirectory(outputDirectory);
}

public final void apply(TemplateDecl templateDecl,
                        Collection subscriptionsCol,
                        Collection subscriptionListsCol,
                        Map allSubscriptionsMap,
                        VelocityContext context)
  throws SiteGenerationException
{
  context.put("subscriptions", subscriptionsCol);
  context.put("subscriptionLists", subscriptionListsCol);
  context.put("allSubscriptionsMap", allSubscriptionsMap);
  context.put("hrefGenerator", hrefGenerator);
  
  Map allSubscriptionDeclMap = new HashMap();
  Iterator subscriptionListColIter = subscriptionListsCol.iterator();
  while(subscriptionListColIter.hasNext())
    {
      SubscriptionListDecl listDecl =
        (SubscriptionListDecl) subscriptionListColIter.next();
      
      allSubscriptionDeclMap.putAll(listDecl);
    }

  context.put("allSubscriptionDeclMap", allSubscriptionDeclMap);
  
  String templateName = templateDecl.getFileName();

  try 
    {
      View view = null;
      if(templateDecl.hasViewClassName())
        {
          String clazzName = templateDecl.getViewClassName();
          Class clazz = Class.forName(clazzName);
          view = (View) clazz.newInstance();
          view.setContext(context);
        }

      doApply(templateDecl,
              subscriptionsCol,
              subscriptionListsCol,
              allSubscriptionsMap,
              view,
              context);
    }
  catch(ResourceNotFoundException ex)
    {
      throw new SiteGenerationException(
        "Cannot find template " + templateName,
        ex);
    }
  catch(ParseErrorException ex)
    {
      throw new SiteGenerationException(
        "Syntax error in template " + templateName,
        ex);
    }
  catch(IOException ex)
    {
      throw new SiteGenerationException(
        "Cannot write output file",
        ex);
    }
  catch(ClassNotFoundException ex)
    {
      log.error("viewClass '" + templateDecl.getViewClassName() +
                "' cannot be found.");
      log.debug(ExceptionUtils.getStackTrace(ex));
    }
  catch(InstantiationException ex)
    {
      log.error("viewClass '" + templateDecl.getViewClassName() +
                "' cannot be instantiated.");
      log.debug(ExceptionUtils.getStackTrace(ex));
    }
  catch(IllegalAccessException ex)
    {
      log.error("viewClass '" + templateDecl.getViewClassName() +
                "' cannot be instantiated.");
      log.debug(ExceptionUtils.getStackTrace(ex));
    }
  catch(Exception ex)
    {
      throw new SiteGenerationException(
        "General Exception during site generation",
        ex);
    }
}

protected abstract void doApply(TemplateDecl templateDecl,
                                Collection subscriptionsCol,
                                Collection subscriptionListsCol,
                                Map allSubscriptionsMap,
                                View view,
                                VelocityContext context)
  throws SiteGenerationException, ResourceNotFoundException,
  ParseErrorException, IOException, Exception;
}


