package org.jaggregator.generator;

import java.util.Date;
import java.util.LinkedList;
import java.util.List;
import org.apache.commons.lang.RandomStringUtils;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.apache.log4j.Category;
import org.jaggregator.rss.util.ChannelRetrievalEvent;
import org.jaggregator.rss.util.ChannelRetrievalListener;
import org.quartz.Job;
import org.quartz.JobDataMap;
import org.quartz.JobDetail;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.quartz.Scheduler;
import org.quartz.SchedulerException;
import org.quartz.SchedulerFactory;
import org.quartz.SimpleTrigger;
import org.quartz.Trigger;
import org.quartz.impl.StdSchedulerFactory;

/**
 * SiteGenerationTrigger.java
 *
 * @author <a href="mailto:jsz@jaggregator.de">Janek Schwarz</a>
 * @version $Revision: 1.3 $
 */
public class SiteGenerationTrigger 
  implements ChannelRetrievalListener
{
private static final String SITE_GENERATION_TRIGGER_NAME =
  "siteGenerationTrigger";

private Category log =
  Category.getInstance(SiteGenerationTrigger.class);
private List retrievedChannelUriList = new LinkedList();
private int channelsBeingRetrieved;
private Scheduler scheduler;
private String groupName;
private SiteGenerator generator;

public SiteGenerationTrigger(SiteGenerator generator) 
  throws SchedulerException
{
  this.generator = generator;
  
  this.groupName = RandomStringUtils.random(8, true, true);
  
  SchedulerFactory schedFact = new StdSchedulerFactory();
  this.scheduler = schedFact.getScheduler();

  scheduler.addJob(createJobDetail(), true);
  
  scheduler.start();
}
  
public synchronized void channelRetrievalStarted(ChannelRetrievalEvent e)
{
  if(channelsBeingRetrieved < 0)
    channelsBeingRetrieved = 0;

  channelsBeingRetrieved++;

  log.debug("channelRetrievalStarted: " + channelsBeingRetrieved);
}

public synchronized void channelRetrievalFinished(ChannelRetrievalEvent e)
{
  channelsBeingRetrieved--;
  
  if(channelsBeingRetrieved < 0)
    channelsBeingRetrieved = 0;
  
  log.debug("channelRetrievalFinished: " + channelsBeingRetrieved);

  if(e.getProgress().equals(ChannelRetrievalEvent.FINISHED_SUCCESS))
    retrievedChannelUriList.add(e.getChannelUri());

  if(channelsBeingRetrieved == 0)
    {
      try
        {
          Trigger trigger = createTrigger();
          scheduler.unscheduleJob(trigger.getName(),
                                  trigger.getGroup());
          
          scheduler.scheduleJob(trigger);
        }
      catch(SchedulerException ex)
        {
          log.error(ex.getMessage());
          log.debug(ExceptionUtils.getStackTrace(ex));
          
          //If we can't delay it, simply start generation
          startSiteGeneration();
        }
    }
}

synchronized void startSiteGeneration()
{
  if(channelsBeingRetrieved > 0)
    return;
  
  try
    {
      generator.generate(retrievedChannelUriList);
    }
  catch(SiteGenerationException ex)
    {
      log.error(ex.getMessage());
      log.debug(ExceptionUtils.getStackTrace(ex));
    }
}

private JobDetail createJobDetail()
{
  JobDetail jobDetail = new JobDetail("siteGenerationTriggerJob",
                                      groupName,
                                      SiteGenerationTrigger.TriggerJob.class);
  jobDetail.setDurability(true);
  jobDetail.getJobDataMap().put("siteGenerationTrigger", this);
  
  return jobDetail;
}

private Trigger createTrigger()
{
  Date startTime = new Date(new Date().getTime() + 10L * 1000);
  Trigger trigger = new SimpleTrigger(SITE_GENERATION_TRIGGER_NAME,
                                      groupName,
                                      "siteGenerationTriggerJob",
                                      groupName,
                                      startTime,
                                      null,
                                      0,
                                      0);
  
  return trigger;
}

public static class TriggerJob
  implements Job
{
private Category log =
  Category.getInstance(TriggerJob.class);

public TriggerJob()
{}

public void execute(JobExecutionContext context)
  throws JobExecutionException
{
  JobDataMap dataMap = context.getJobDetail().getJobDataMap();
  SiteGenerationTrigger generationTrigger =
    (SiteGenerationTrigger) dataMap.get("siteGenerationTrigger");

  generationTrigger.startSiteGeneration();
}
}
}
