package org.jaggregator.rss.common;

import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.Map;
import org.apache.commons.lang.StringUtils;
import org.apache.ecs.html.Comment;
import org.apache.ecs.xml.XML;
import org.apache.ecs.xml.XMLDocument;
import org.jaggregator.rss.Channel;
import org.jaggregator.rss.ChannelExporter;
import org.jaggregator.rss.Item;
import org.jaggregator.rss.RssException;
import org.jaggregator.rss.Subscription;
import org.jaggregator.rss.SubscriptionExporter;
import org.jaggregator.rss.exporter.Commons;
import org.jaggregator.rss.util.ExporterFactory;
import org.jaggregator.rss.util.FileUtils;

/**
 * Implementation of the <code>SubscriptionExporter</code>
 * interface. Given a directory,
 * <code>SubscriptionExporterImpl</code> create a uniquely named
 * subdirectory and writes files for the channel and subscription
 * data in that subdirectory. Both files contain XML
 * representations of the data.
 *
 * @author <a href="mailto:jsz@jaggregator.org">Janek Schwarz</a>
 * @version $Revision: 1.11 $
 */
public class SubscriptionExporterImpl
 implements SubscriptionExporter
{
static final String SUBSCRIPTION_FILE_NAME = "subscription.xml";
static final String CHANNEL_FILE_NAME = "channel.xml";

private DateFormat format;
private File parentDirectory;

/**
 * Create a new <code>SubscriptionExporterImpl</code> instance.
 *
 * @param parentDirectory a directory under which the
 * subscription data should be stored. If the directory does not
 * exist, it will be created later.

 * @exception IllegalArgumentException if
 * <code>parentDirectory</code> exists but is not a directory
 */
public SubscriptionExporterImpl(File parentDirectory)
{
  if(parentDirectory.exists() && parentDirectory.isDirectory() == false)
    throw new IllegalArgumentException(
      "argument \"parentDirectory\" is not a directory");

  this.parentDirectory = parentDirectory;

  format = (DateFormat) Commons.DATE_FORMAT.clone();
}

public void export(Subscription subscription)
  throws RssException, IOException
{
  if((subscription instanceof SubscriptionImpl) == false)
    throw new IllegalArgumentException(
      "Can only export SubscriptionImpl instances");

  File subscriptionDirectory =
    createSubscriptionDirectoryObject(subscription);
  writeSubscriptionFile(subscription, subscriptionDirectory);

  Channel channel = subscription.getChannel();

  if(channel != null)
    writeChannelFile(channel,
                     subscriptionDirectory);
}

private File createSubscriptionDirectoryObject(Subscription subscription)
  throws IOException
{
  String subscriptionDirectoryName =
    FileUtils.uriToFileName(subscription.getUrl().toString());
  File subscriptionDirectory = new File(parentDirectory,
                                        subscriptionDirectoryName);
  if(subscriptionDirectory.exists() == false)
    {
      boolean success = subscriptionDirectory.mkdirs();
      if(success == false)
        throw new IOException("Can't create subscription directory: " +
                              subscriptionDirectoryName);
    }

  return subscriptionDirectory;
}

private void writeSubscriptionFile(Subscription subscription,
                                   File subscriptionDirectory)
  throws IOException
{
  XMLDocument document = createSubscriptionXMLDocument(subscription);
  File subscriptionFile = new File(subscriptionDirectory,
                                   SUBSCRIPTION_FILE_NAME);

  PrintWriter subscriptionWriter =
    new PrintWriter(FileUtils.createWriter(subscriptionFile));

  document.output(subscriptionWriter);
  subscriptionWriter.close();
}


XMLDocument createSubscriptionXMLDocument(Subscription subscription)
{
  XMLDocument document = new XMLDocument(1.0);
  document.setCodeset("UTF-8");

  XML subscriptionXML = new XML("subscription");
  subscriptionXML.addXMLAttribute("version", "1.0");
  document.addElement(subscriptionXML);

  subscriptionXML.addElement(
    new XML("url").addElement(subscription.getUrl().toString()));

  if(subscription.getSubscriptionDate() != null)
    {
      subscriptionXML.addElement(
        new XML("subscriptionDate").addElement(
          format.format(subscription.getSubscriptionDate())));
    }

  if(subscription.getLastRetrievedDate() != null)
    {
      subscriptionXML.addElement(
        new XML("lastRetrievedDate").addElement(
          format.format(subscription.getLastRetrievedDate())));
    }

  subscriptionXML.addElement(
    new XML("channelLevelChanges").addElement(
      String.valueOf(subscription.hasChannelLevelChanges())));

  if(subscription.hasLastModifiedString())
    {
      subscriptionXML.addElement(
        new XML("lastModifiedString").addElement(
          subscription.getLastModifiedString()));
    }

  if(subscription.hasEtagString())
    {
      subscriptionXML.addElement(
        new XML("eTagString").addElement(
          subscription.getEtagString()));
    }

  if(subscription.getChannel() != null &&
     subscription.getChannel().hasItems())
    {
      XML channelInfosXML = new XML("channelInfos");
      subscriptionXML.addElement(channelInfosXML);

      channelInfosXML.addElement(
        new XML("channelVersion").addElement(
          String.valueOf(subscription.getChannel().getVersion().getName())));

      XML dateItemListXML = new XML("dateItemList");
      channelInfosXML.addElement(dateItemListXML);

      Iterator dateItemIterator =
        ((SubscriptionImpl) subscription).dateItemMap.entrySet().iterator();

      while(dateItemIterator.hasNext())
        {
          Map.Entry entry = (Map.Entry) dateItemIterator.next();

          Date date = (Date) entry.getKey();
          ArrayList itemList = (ArrayList) entry.getValue();

          Iterator itemIter = itemList.iterator();
          while(itemIter.hasNext())
            {
              Item item = (Item) itemIter.next();

              XML dateItemEntry = new XML("dateItemEntry");
              dateItemListXML.addElement(dateItemEntry);

              if(item.hasTitle())
                {
                  Comment comment =
                    new Comment(StringUtils.replace(item.getTitle(),
                                                    "-",
                                                    "\\-"));
                  dateItemEntry.addElement(comment);
                }

              dateItemEntry.addElement(
                new XML("date").addElement(
                  format.format(date)));
              dateItemEntry.addElement(
                new XML("hash").addElement(String.valueOf(item.hashCode())));
            }
        }
    }

  return document;
}

private void writeChannelFile(Channel channel,
                              File subscriptionDirectory)
  throws IOException, RssException
{
  ExporterFactory factory = ExporterFactory.getInstance();
  File channelFile = new File(subscriptionDirectory,
                              CHANNEL_FILE_NAME);

  ChannelExporter exporter = factory.newExporter(channel.getVersion(),
                                                 channelFile);
  exporter.export(channel);
}
}

