package org.jaggregator.rss.common;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedList;
import org.apache.commons.digester.Digester;
import org.apache.commons.digester.RuleSet;
import org.jaggregator.rss.Channel;
import org.jaggregator.rss.Converter;
import org.jaggregator.rss.ConverterSet;
import org.jaggregator.rss.Parser;
import org.jaggregator.rss.RssException;
import org.jaggregator.rss.Subscription;
import org.jaggregator.rss.SubscriptionParser;
import org.jaggregator.rss.Version;
import org.jaggregator.rss.convert.DateConverter;
import org.jaggregator.rss.convert.UrlConverter;
import org.jaggregator.rss.convert.VersionConverter;
import org.jaggregator.rss.util.FileUtils;
import org.jaggregator.rss.util.ParserFactory;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 * Reads a subscription from disk. It is able to create
 * <code>SubscriptionImpl</code> objects that have been
 * externalized using a <code>SubscriptionExporterImpl</code>.
 *
 * @author <a href="mailto:jsz@jaggregator.org">Janek Schwarz</a>
 * @version $Revision: 1.5 $
 */
public class SubscriptionParserImpl 
  extends DigesterSupport
  implements SubscriptionParser
{
private File subscriptionsRoot;

/**
 * Package-private for testability.
 */
ChannelInfos channelInfos;

public SubscriptionParserImpl(File subscriptionsRoot) 
  throws FileNotFoundException, IOException
{
  if(subscriptionsRoot.exists() == false)
    throw new FileNotFoundException(
      subscriptionsRoot.getAbsolutePath() + " does not exist");
  else if(subscriptionsRoot.isDirectory() == false)
    throw new IOException(
      subscriptionsRoot.getAbsolutePath() + " is not a directory");

  this.subscriptionsRoot = subscriptionsRoot;

  setup();
}

protected ConverterSet getConverterSet()
{
  return new ConverterSet()
    {
    public Converter[] getConverters()
    {
      return new Converter[]
        {
          new UrlConverter(),
          new DateConverter(),
          new VersionConverter()
        };
    }
    };
}

protected Collection getDigesterRuleSetCollection()
{
  LinkedList list = new LinkedList();
  list.add(new SubscriptionRuleSet());
  
  return list;
}

public Subscription parse(URL channelUri)
  throws RssException
{
  try
    {
      RootObject rootObject = 
        parseSubscriptionFile(channelUri);

      SubscriptionImpl subscription = rootObject.subscription;
      channelInfos = rootObject.channelInfos;
      if(channelInfos != null)
        {
          Channel channel = parseChannelFile(channelUri,
                                             channelInfos);
          HashMap itemDateMap = channelInfos.itemDateMap;
          
          subscription.setChannel(channel, itemDateMap);
        }
      
      return subscription;
    }
  catch(FileNotFoundException ex)
    {
      throw new RssException(ex);
    }
  catch(IOException ex)
    {
      throw new RssException(ex);
    }
  catch(SAXException ex)
    {
      throw new RssException(ex);
    }
}

private RootObject parseSubscriptionFile(URL channelUri)
  throws SAXException, IOException
{
  InputSource isource = 
    createSubscriptionFileIsource(channelUri);

  digester.push(new RootObject());
  
  RootObject rootObject = 
    (RootObject) digester.parse(isource);

  return rootObject;
}

private Channel parseChannelFile(URL channelUri, ChannelInfos channelInfos)
  throws RssException, FileNotFoundException
{
  InputSource isource = createChannelFileIsource(channelUri);
  
  Parser parser = 
    ParserFactory.getInstance().newParser(channelInfos.version);
  
  return parser.parse(isource);
}

InputSource createChannelFileIsource(URL channelUri)
  throws FileNotFoundException
{
  return createInputSource(channelUri,
                           SubscriptionExporterImpl.CHANNEL_FILE_NAME);
}

InputSource createSubscriptionFileIsource(URL channelUri)
  throws FileNotFoundException
{
  return createInputSource(channelUri,
                           SubscriptionExporterImpl.SUBSCRIPTION_FILE_NAME);
}

InputSource createInputSource(URL channelUri, String fileName)
  throws FileNotFoundException
{
  String subscriptionDirectoryName = 
    FileUtils.uriToFileName(channelUri.toString());
  File subscriptionDirectory =
    new File(subscriptionsRoot,
             subscriptionDirectoryName);
  File file = 
    new File(subscriptionDirectory,
             fileName);

  InputStream istream = 
    new BufferedInputStream(
      new FileInputStream(file));
  
  InputSource isource = new InputSource(istream);
  isource.setSystemId(file.getAbsolutePath());

  return isource;
}

private class RootObject
{
SubscriptionImpl subscription;
ChannelInfos channelInfos;

public void setSubscription(SubscriptionImpl subscription)
{
  this.subscription = subscription;
}

public void setChannelInfos(ChannelInfos channelInfos)
{
  this.channelInfos = channelInfos;
}
}

private class SubscriptionRuleSet
  implements RuleSet
{
/**
 * As the XML definition of <code>SubscriptionImpl</code> does
 * not contain namespaces, this method returns <code>null</code>.
 */
public String getNamespaceURI()
{
  return null;
}

public void addRuleInstances(Digester digester)
{
  digester.addObjectCreate("subscription", 
                           "org.jaggregator.rss.common.SubscriptionImpl");
  digester.addSetNext("subscription", "setSubscription");
  digester.addCallMethod("subscription/url", 
                         "setUrl", 
                         0,
                         new Class[] { URL.class });
  digester.addCallMethod("subscription/subscriptionDate", 
                         "setSubscriptionDate", 
                         0,
                         new Class[] { Date.class });
  digester.addCallMethod("subscription/lastRetrievedDate", 
                         "setLastRetrievedDate", 
                         0,
                         new Class[] { Date.class });
  digester.addCallMethod("subscription/channelLevelChanges", 
                         "setChannelLevelChanges", 
                         0,
                         new Class[] { Boolean.TYPE });
  digester.addObjectCreate("subscription/channelInfos",
                           ChannelInfos.class);
  digester.addSetRoot("subscription/channelInfos",
                      "setChannelInfos");
  digester.addCallMethod("subscription/channelInfos/channelVersion",
                         "setVersion",
                         0,
                         new Class[] { Version.class });
  digester.addObjectCreate("subscription/channelInfos/dateItemList",
                           "java.util.HashMap");
  digester.addSetNext("subscription/channelInfos/dateItemList",
                      "setItemDateMap");
  digester.addCallMethod(
    "subscription/channelInfos/dateItemList/dateItemEntry",
    "put",
    2,
    new Class[] { Integer.class, Date.class });
  digester.addCallParam(
    "subscription/channelInfos/dateItemList/dateItemEntry/hash",
    0);
  digester.addCallParam(
    "subscription/channelInfos/dateItemList/dateItemEntry/date",
    1);
}
}
}
