package org.jaggregator.rss.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.io.Writer;
import java.net.URLDecoder;
import java.net.URLEncoder;

/**
 * FileUtils provides several utility methods for files, streams
 * and readers/writers. All delete, copy and stream methods are
 * borrowed from Avalon.
 *
 * @author <a href="mailto:jsz@jaggregator.org">Janek Schwarz</a>
 * @version $Revision: 1.5 $
 */
public class FileUtils 
{
private static final int DEFAULT_BUFFER_SIZE = 1024 * 4;

public static String uriToFileName(String uriString)
{
  return URLEncoder.encode(uriString);
}

public static String fileNameToUri(String fileName)
{
  return URLDecoder.decode(fileName);
}

public static Writer createWriter(File file)
  throws FileNotFoundException
{
  try
    {
      return createWriter(file, "UTF-8");
    }
  catch(UnsupportedEncodingException ex)
    {
      //can't happen, since UTF8 is always supported.
    }

  return null;
}

public static Writer createWriter(File file, String charsetString)
  throws UnsupportedEncodingException, FileNotFoundException
{
  OutputStream ostream = new FileOutputStream(file);
  return new OutputStreamWriter(ostream,
                                charsetString);
}

/**
 * Recursively delete a directory.
 */
public static void deleteDirectory( final String directory )
  throws IOException
{
  deleteDirectory( new File( directory ) );
}

/**
 * Recursively delete a directory.
 */
public static void deleteDirectory( final File directory )
  throws IOException
{
  if( !directory.exists() )
    {
      return;
    }

  cleanDirectory( directory );
  if( !directory.delete() )
    {
      final String message =
        "Directory " + directory + " unable to be deleted.";
      throw new IOException( message );
    }
}

/**
 * Clean a directory without deleting it.
 */
public static void cleanDirectory( final File directory )
  throws IOException
{
  if( !directory.exists() )
    {
      final String message = directory + " does not exist";
      throw new IllegalArgumentException( message );
    }
  
  if( !directory.isDirectory() )
    {
      final String message = directory + " is not a directory";
      throw new IllegalArgumentException( message );
    }
  
  IOException exception = null;
  
  final File[] files = directory.listFiles();
  for( int i = 0; i < files.length; i++ )
    {
      final File file = files[ i ];
      try
        {
          forceDelete( file );
        }
      catch( final IOException ioe )
        {
          exception = ioe;
        }
    }

  if( null != exception )
    {
      throw exception;
    }
}

/**
 * Delete a file. If file is directory delete it and all sub-directories.
 */
public static void forceDelete( final File file )
  throws IOException
{
  if( file.isDirectory() )
    {
      deleteDirectory( file );
    }
  else
    {
      if( !file.delete() )
        {
          final String message =
            "File " + file + " unable to be deleted.";
          throw new IOException( message );
        }
    }
}

/**
 * Copy bytes from an <code>InputStream</code> to an <code>OutputStream</code>.
 */
public static void copy( final InputStream input, final OutputStream output )
  throws IOException
{
  copy( input, output, DEFAULT_BUFFER_SIZE );
}

/**
 * Copy bytes from an <code>InputStream</code> to an <code>OutputStream</code>.
 * @param bufferSize Size of internal buffer to use.
 */
public static void copy(final InputStream input,
                        final OutputStream output,
                        final int bufferSize )
  throws IOException
{
  final byte[] buffer = new byte[ bufferSize ];
  int n = 0;
  while( -1 != ( n = input.read( buffer ) ) )
    {
      output.write( buffer, 0, n );
    }
}

/**
 * Unconditionally close an <code>OutputStream</code>.
 * Equivalent to {@link OutputStream#close()}, except any
 * exceptions will be ignored.
 * @param output A (possibly null) OutputStream
 */
public static void shutdownStream( final OutputStream output )
{
  if( output == null )
    {
      return;
    }
  
  try
    {
      output.close();
    }
  catch( final IOException ioe )
    {
    }
}

/**
 * Unconditionally close an <code>InputStream</code>.  Equivalent
 * to {@link InputStream#close()}, except any exceptions will be
 * ignored.
 * @param input A (possibly null) InputStream
 */
public static void shutdownStream( final InputStream input )
{
  if( input == null )
    {
      return;
    }
  
  try
    {
      input.close();
    }
  catch( final IOException ioe )
    {
    }
}

/**
 * Copy file from source to destination. If
 * <code>destinationDirectory</code> does not exist, it (and any
 * parent directories) will be created. If a file
 * <code>source</code> in <code>destinationDirectory</code>
 * exists, it will be overwritten.
 *
 * @param source An existing <code>File</code> to copy.
 * @param destinationDirectory A directory to copy
 * <code>source</code> into.
 *
 * @throws FileNotFoundException if <code>source</code> isn't a
 * normal file.
 * @throws IllegalArgumentException if
 * <code>destinationDirectory</code> isn't a directory.
 * @throws IOException if <code>source</code> does not exist, the
 * file in <code>destinationDirectory</code> cannot be written
 * to, or an IO error occurs during copying.
 */
public static void copyFileToDirectory( final String source,
                                        final String destinationDirectory )
  throws IOException
{
  copyFileToDirectory( new File( source ),
                       new File( destinationDirectory ) );
}

/**
 * Copy file from source to destination. If
 * <code>destinationDirectory</code> does not exist, it (and any
 * parent directories) will be created. If a file
 * <code>source</code> in <code>destinationDirectory</code>
 * exists, it will be overwritten.
 *
 * @param source An existing <code>File</code> to copy.
 * @param destinationDirectory A directory to copy
 * <code>source</code> into.
 *
 * @throws FileNotFoundException if <code>source</code> isn't a
 * normal file.
 * @throws IllegalArgumentException if
 * <code>destinationDirectory</code> isn't a directory.
 * @throws IOException if <code>source</code> does not exist, the
 * file in <code>destinationDirectory</code> cannot be written
 * to, or an IO error occurs during copying.
 */
public static void copyFileToDirectory( final File source,
                                        final File destinationDirectory )
  throws IOException
{
  if( destinationDirectory.exists() && !destinationDirectory.isDirectory() )
    {
      throw new IllegalArgumentException( "Destination is not a directory" );
    }

  copyFile( source, new File( destinationDirectory, source.getName() ) );
}

/**
 * Copy file from source to destination. The directories up to
 * <code>destination</code> will be created if they don't already
 * exist. <code>destination</code> will be overwritten if it
 * already exists.
 *
 * @param source An existing non-directory <code>File</code> to
 * copy bytes from.
 * @param destination A non-directory <code>File</code> to write
 * bytes to (possibly overwriting).
 *
 * @throws IOException if <code>source</code> does not exist,
 * <code>destination</code> cannot be written to, or an IO error
 * occurs during copying.
 *
 * @throws FileNotFoundException if <code>destination</code> is a
 * directory (use {@link #copyFileToDirectory}).
 */
public static void copyFile( final File source, final File destination )
  throws IOException
{
  //check source exists
  if( !source.exists() )
    {
      final String message = "File " + source + " does not exist";
      throw new IOException( message );
    }

  //does destinations directory exist ?
  if( destination.getParentFile() != null &&
      !destination.getParentFile().exists() )
    {
      destination.getParentFile().mkdirs();
    }

  //make sure we can write to destination
  if( destination.exists() && !destination.canWrite() )
    {
      final String message = "Unable to open file " +
        destination + " for writing.";
      throw new IOException( message );
    }

  final FileInputStream input = new FileInputStream( source );
  final FileOutputStream output = new FileOutputStream( destination );
  copy( input, output );
  shutdownStream( input );
  shutdownStream( output );

  if( source.length() != destination.length() )
    {
      final String message = "Failed to copy full contents from " + source +
        " to " + destination;
      throw new IOException( message );
    }
}
}
