/*
 *  Copyright James House (c) 2001-2002, PartNET Inc. 2002
 *
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
package org.jaggregator.util;

import java.util.Calendar;
import java.util.Date;

/**
 * <p>Convenience and utility methods for simplifying the
 * construction of <code>Date</code> objects.</p>
 *
 * <p>This code is based on <a
 * href="http://www.sourceforge.net/projects/quartz">Quartz</a>.</p>
 *
 * @author James House
 */
public class DateUtils
{

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 * Constants.
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

public static final int SUNDAY = 1;
public static final int MONDAY = 2;
public static final int TUESDAY = 3;
public static final int WEDNESDAY = 4;
public static final int THURSDAY = 5;
public static final int FRIDAY = 6;
public static final int SATURDAY = 7;

public static final int LAST_DAY_OF_MONTH = -1;

public static final long MILLISECONDS_IN_MINUTE = 60l * 1000l;
public static final long MILLISECONDS_IN_HOUR = 60l * 60l * 1000l;
public static final long SECONDS_IN_DAY = 24l * 60l * 60L;
public static final long MILLISECONDS_IN_DAY = SECONDS_IN_DAY * 1000l;

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 * Interface.
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

private static void validateDayOfWeek(int dayOfWeek) {
  if (dayOfWeek < SUNDAY || dayOfWeek > SATURDAY)
    throw new IllegalArgumentException("Invalid day of week.");
}

private static void validateHour(int hour) {
  if (hour < 0 || hour > 23)
    throw new IllegalArgumentException("Invalid hour (must be >= 0 and <= 23).");
}

private static void validateMinute(int minute) {
  if (minute < 0 || minute > 59)
    throw new IllegalArgumentException("Invalid minute (must be >= 0 and <= 59).");
}

private static void validateSecond(int second) {
  if (second < 0 || second > 59)
    throw new IllegalArgumentException("Invalid second (must be >= 0 and <= 59).");
}

private static void validateDayOfMonth(int day) {
  if ((day < 0 || day > 31) && day != LAST_DAY_OF_MONTH)
    throw new IllegalArgumentException("Invalid day of month.");
}

private static void validateMonth(int month) {
  if (month < 1 || month > 12)
    throw new IllegalArgumentException("Invalid month (must be >= 1 and <= 12.");
}

private static void validateYear(int year) {
  if (year < 1970 || year > 2099)
    throw new IllegalArgumentException("Invalid year (must be >= 1970 and <= 2099.");
}

/**
 * <p>Returns a date that is rounded to the next even day above the given
 * date.</p>
 *
 * <p>For example an input date with a time of 05/20/2003,
 * 08:13:54 would result in a date with the time of 05/21/2003
 * 00:00:00.
 *
 * @param date the Date to round, if <code>null</code> the current time will
 * be used
 */
public static Date getEvenDayDate(Date date)
{
  if(date == null)
    date = new Date();

  Calendar c = Calendar.getInstance();
  c.setTime(date);
  c.setLenient(true);

  c.set(Calendar.HOUR_OF_DAY, 0);
  c.set(Calendar.MINUTE, 0);
  c.set(Calendar.SECOND, 0);
  c.set(Calendar.MILLISECOND, 0);

  c.add(Calendar.DATE, 1);
  
  return c.getTime();
}

/**
 * <p>Returns a date that is rounded to the next even day below the given
 * date.</p>
 *
 * <p>For example an input date with a time of 05/20/2003,
 * 08:13:54 would result in a date with the time of 05/20/2003
 * 00:00:00.
 *
 * @param date the Date to round, if <code>null</code> the
 * current time will be used
 */
public static Date getEvenDayDateBefore(Date date)
{
  if(date == null)
    date = new Date();

  Calendar c = Calendar.getInstance();
  c.setTime(date);
  c.setLenient(true);

  c.set(Calendar.HOUR_OF_DAY, 0);
  c.set(Calendar.MINUTE, 0);
  c.set(Calendar.SECOND, 0);
  c.set(Calendar.MILLISECOND, 0);
  
  return c.getTime();
}

/**
 * <p>Returns a date that is rounded to the next even hour above the given
 * date.</p>
 *
 * <p>For example an input date with a time of 08:13:54 would result in a
 * date with the time of 09:00:00.  If the date's time is in the 23rd hour,
 * the date's 'day' will be promoted, and the time will be set to
 * 00:00:00.</p>
 *
 * @param date the Date to round, if <code>null</code> the current time will
 * be used
 */
public static Date getEvenHourDate(Date date)
{
  if (date == null)
    date = new Date();

  Calendar c = Calendar.getInstance();
  c.setTime(date);
  c.setLenient(true);

  c.set(Calendar.HOUR_OF_DAY, c.get(Calendar.HOUR_OF_DAY) + 1);
  c.set(Calendar.MINUTE, 0);
  c.set(Calendar.SECOND, 0);
  c.set(Calendar.MILLISECOND, 0);

  return c.getTime();
}

/**
 * <p>Returns a date that is rounded to the previous even hour below the given
 * date.</p>
 *
 * <p>For example an input date with a time of 08:13:54 would result in a
 * date with the time of 08:00:00.</p>
 *
 * @param date the Date to round, if <code>null</code> the current time will
 * be used
 */
public static Date getEvenHourDateBefore(Date date)
{
  if (date == null)
    date = new Date();

  Calendar c = Calendar.getInstance();
  c.setTime(date);

  c.set(Calendar.MINUTE, 0);
  c.set(Calendar.SECOND, 0);
  c.set(Calendar.MILLISECOND, 0);

  return c.getTime();
}

/**
 * <p>Returns a date that is rounded to the next even hour above the given
 * date.</p>
 *
 * <p>For example an input date with a time of 08:13:54 would result in a
 * date with the time of 08:14:00.  If the date's time is in the 59th minute,
 * then the hour (and possibly the day) will be promoted.</p>
 *
 * @param date the Date to round, if <code>null</code> the current time will
 * be used
 */
public static Date getEvenMinuteDate(Date date)
{
  if (date == null)
    date = new Date();

  Calendar c = Calendar.getInstance();
  c.setTime(date);
  c.setLenient(true);

  c.set(Calendar.MINUTE, c.get(Calendar.MINUTE) + 1);
  c.set(Calendar.SECOND, 0);
  c.set(Calendar.MILLISECOND, 0);

  return c.getTime();
}

/**
 * <p>Returns a date that is rounded to the previous even minute
 * below the given date.</p>
 *
 * <p>For example an input date with a time of 08:13:54 would result in a
 * date with the time of 08:13:00.</p>
 *
 * @param date the Date to round, if <code>null</code> the current time will
 * be used
 */
public static Date getEvenMinuteDateBefore(Date date)
{
  if (date == null)
    date = new Date();

  Calendar c = Calendar.getInstance();
  c.setTime(date);

  c.set(Calendar.SECOND, 0);
  c.set(Calendar.MILLISECOND, 0);

  return c.getTime();
}

/**
 * <p>Returns a date that is rounded to the next even second above the given
 * date.</p>
 *
 * @param date the Date to round, if <code>null</code> the current time will
 * be used
 */
public static Date getEvenSecondDate(Date date)
{
  if (date == null)
    date = new Date();

  Calendar c = Calendar.getInstance();
  c.setTime(date);
  c.setLenient(true);

  c.set(Calendar.SECOND, c.get(Calendar.SECOND) + 1);
  c.set(Calendar.MILLISECOND, 0);

  return c.getTime();
}

/**
 * <p>Returns a date that is rounded to the previous even second below the
 * given date.</p>
 *
 * <p>For example an input date with a time of 08:13:54.341 would result in a
 * date with the time of 08:13:00.000.</p>
 *
 * @param date the Date to round, if <code>null</code> the current time will
 * be used
 */
public static Date getEvenSecondDateBefore(Date date)
{
  if (date == null)
    date = new Date();
  
  Calendar c = Calendar.getInstance();
  c.setTime(date);
  
  c.set(Calendar.MILLISECOND, 0);

  return c.getTime();
}

/**
 * <p>Returns a date that is rounded to the next even multiple of the given
 * minute.</p>
 *
 * <p>For example an input date with a time of 08:13:54, and an input
 * minute-base of 5 would result in a date with the time of 08:15:00.
 * The same input date with an input minute-base of 10 would result in a
 * date with the time of 08:20:00. But a date with the time 08:53:31 and an
 * input minute-base of 45 would result in 09:00:00, because the even-hour
 * is the next 'base' for 45-minute intervals.</p>
 *
 * <p>More examples:
 * <table>
 *  <tr>
 *   <th>Input Time</th>
 *   <th>Minute-Base</th>
 *   <th>Result Time</th>
 *  </tr>
 *  <tr>
 *   <td>11:16:41</td>
 *   <td>20</td>
 *   <td>11:20:00</td>
 *  </tr>
 *  <tr>
 *   <td>11:36:41</td>
 *   <td>20</td>
 *   <td>11:40:00</td>
 *  </tr>
 *  <tr>
 *   <td>11:46:41</td>
 *   <td>20</td>
 *   <td>12:00:00</td>
 *  </tr>
 *  <tr>
 *   <td>11:26:41</td>
 *   <td>30</td>
 *   <td>11:30:00</td>
 *  </tr>
 *  <tr>
 *   <td>11:36:41</td>
 *   <td>30</td>
 *   <td>12:00:00</td>
 *  </tr>
 *   <td>11:16:41</td>
 *   <td>17</td>
 *   <td>11:17:00</td>
 *  </tr>
 *  </tr>
 *   <td>11:17:41</td>
 *   <td>17</td>
 *   <td>11:34:00</td>
 *  </tr>
 *  </tr>
 *   <td>11:52:41</td>
 *   <td>17</td>
 *   <td>12:00:00</td>
 *  </tr>
 *  </tr>
 *   <td>11:52:41</td>
 *   <td>5</td>
 *   <td>11:55:00</td>
 *  </tr>
 *  </tr>
 *   <td>11:57:41</td>
 *   <td>5</td>
 *   <td>12:00:00</td>
 *  </tr>
 *  </tr>
 *   <td>11:17:41</td>
 *   <td>0</td>
 *   <td>12:00:00</td>
 *  </tr>
 *  </tr>
 *   <td>11:17:41</td>
 *   <td>1</td>
 *   <td>11:08:00</td>
 *  </tr>
 * </table>
 * </p>
 *
 * @param date the Date to round, if <code>null</code> the current time will
 * be used
 * @param minuteBase the base-minute to set the time on
 *
 * @see #getNextGivenSecondDate(Date, int)
 */
public static Date getNextGivenMinuteDate(Date date, int minuteBase)
{
  if (minuteBase < 0 || minuteBase > 59)
    throw new IllegalArgumentException("minuteBase must be >=0 and <= 59");
  
  if (date == null)
    date = new Date();
  
  Calendar c = Calendar.getInstance();
  c.setTime(date);
  c.setLenient(true);

  if (minuteBase == 0)
    {
      c.set(Calendar.HOUR_OF_DAY, c.get(Calendar.HOUR_OF_DAY) + 1);
      c.set(Calendar.MINUTE, 0);
      c.set(Calendar.SECOND, 0);
      c.set(Calendar.MILLISECOND, 0);
      
      return c.getTime();
    }

  int minute = c.get(Calendar.MINUTE);

  int arItr = minute / minuteBase;

  int nextMinuteOccurance = minuteBase * (arItr + 1);

  if (nextMinuteOccurance < 60)
    {
      c.set(Calendar.MINUTE, nextMinuteOccurance);
      c.set(Calendar.SECOND, 0);
      c.set(Calendar.MILLISECOND, 0);
      
      return c.getTime();
    }
  else
    {
      c.set(Calendar.HOUR_OF_DAY, c.get(Calendar.HOUR_OF_DAY) + 1);
      c.set(Calendar.MINUTE, 0);
      c.set(Calendar.SECOND, 0);
      c.set(Calendar.MILLISECOND, 0);
      
      return c.getTime();
    }
}

/**
 * <p>Returns a date that is rounded to the next even multiple of the given
 * minute.</p>
 *
 * <p>The rules for calculating the second are the same as those for
 * calculating the minute in the method
 * <code>getNextGivenMinuteDate(..)<code>.</p>
 *
 * @param date the Date to round, if <code>null</code> the current time will
 * be used
 * @param secondBase the base-second to set the time on
 *
 * @see #getNextGivenMinuteDate(Date, int)
 */
public static Date getNextGivenSecondDate(Date date, int secondBase)
{
  if (secondBase < 0 || secondBase > 59)
    throw new IllegalArgumentException("secondBase must be >=0 and <= 59");

  if (date == null)
    date = new Date();

  Calendar c = Calendar.getInstance();
  c.setTime(date);
  c.setLenient(true);

  if (secondBase == 0)
    {
      c.set(Calendar.MINUTE, c.get(Calendar.MINUTE) + 1);
      c.set(Calendar.SECOND, 0);
      c.set(Calendar.MILLISECOND, 0);
      
      return c.getTime();
    }

  int second = c.get(Calendar.SECOND);

  int arItr = second / secondBase;

  int nextSecondOccurance = secondBase * (arItr + 1);

  if (nextSecondOccurance < 60)
    {
      c.set(Calendar.SECOND, nextSecondOccurance);
      c.set(Calendar.MILLISECOND, 0);
      
      return c.getTime();
    }
  else
    {
      c.set(Calendar.MINUTE, c.get(Calendar.MINUTE) + 1);
      c.set(Calendar.SECOND, 0);
      c.set(Calendar.MILLISECOND, 0);
      
      return c.getTime();
    }
}

/**
 * <p>Get a <code>Date</code> object that represents the given time, on 
 * today's date.</p>
 * 
 * @param second The value (0-59) to give the seconds field of the date
 * @param minute The value (0-59) to give the minutes field of the date
 * @param hour The value (0-23) to give the hours field of the date
 */
public static Date getDateOf(int second, int minute, int hour)
{
  validateSecond(second);
  validateMinute(minute);
  validateHour(hour);

  Date date = new Date();

  Calendar c = Calendar.getInstance();
  c.setTime(date);
  c.setLenient(true);

  c.set(Calendar.HOUR_OF_DAY, hour);
  c.set(Calendar.MINUTE, minute);
  c.set(Calendar.SECOND, second);
  c.set(Calendar.MILLISECOND, 0);

  return c.getTime();
}

/**
 * <p>Get a <code>Date</code> object that represents the given time, on 
 * the given date.</p>
 * 
 * @param second The value (0-59) to give the seconds field of the date
 * @param minute The value (0-59) to give the minutes field of the date
 * @param hour The value (0-23) to give the hours field of the date
 * @param dayOfMonth The value (1-31) to give the day of month field of the date
 * @param month The value (1-12) to give the month field of the date
 */
public static Date getDateOf(int second,
                             int minute,
                             int hour,
                             int dayOfMonth,
                             int month)
{
  validateSecond(second);
  validateMinute(minute);
  validateHour(hour);
  validateDayOfMonth(dayOfMonth);
  validateMonth(month);

  Date date = new Date();

  Calendar c = Calendar.getInstance();
  c.setTime(date);

  c.set(Calendar.MONTH, month - 1);
  c.set(Calendar.DAY_OF_MONTH, dayOfMonth);
  c.set(Calendar.HOUR_OF_DAY, hour);
  c.set(Calendar.MINUTE, minute);
  c.set(Calendar.SECOND, second);
  c.set(Calendar.MILLISECOND, 0);

  return c.getTime();
}

/**
 * <p>Get a <code>Date</code> object that represents the given time, on 
 * the given date.</p>
 * 
 * @param second The value (0-59) to give the seconds field of the date
 * @param minute The value (0-59) to give the minutes field of the date
 * @param hour The value (0-23) to give the hours field of the date
 * @param dayOfMonth The value (1-31) to give the day of month field of the date
 * @param month The value (1-12) to give the month field of the date
 * @param year The value (1970-2099) to give the year field of the date
 */
public static Date getDateOf(int second,
                             int minute,
                             int hour,
                             int dayOfMonth,
                             int month,
                             int year)
{
  validateSecond(second);
  validateMinute(minute);
  validateHour(hour);
  validateDayOfMonth(dayOfMonth);
  validateMonth(month);
  validateYear(year);

  Date date = new Date();

  Calendar c = Calendar.getInstance();
  c.setTime(date);

  c.set(Calendar.YEAR, year);
  c.set(Calendar.MONTH, month - 1);
  c.set(Calendar.DAY_OF_MONTH, dayOfMonth);
  c.set(Calendar.HOUR_OF_DAY, hour);
  c.set(Calendar.MINUTE, minute);
  c.set(Calendar.SECOND, second);
  c.set(Calendar.MILLISECOND, 0);

  return c.getTime();
}
}
