/*
 * (C) Copyright 2002-2003, Andy Clark.  All rights reserved.
 *
 * This file is distributed under an Apache style license. Please
 * refer to the LICENSE file for specific details.
 */
package org.jaggregator.util;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import org.apache.commons.collections.Predicate;
import org.apache.log4j.Category;
import org.apache.xerces.xni.Augmentations;
import org.apache.xerces.xni.NamespaceContext;
import org.apache.xerces.xni.QName;
import org.apache.xerces.xni.XMLAttributes;
import org.apache.xerces.xni.XMLLocator;
import org.apache.xerces.xni.XMLResourceIdentifier;
import org.apache.xerces.xni.XMLString;
import org.apache.xerces.xni.XNIException;
import org.cyberneko.html.filters.DefaultFilter;

/**
 * This class is a document filter capable of removing specified
 * elements from the processing stream. There are three options for
 * processing document elements:
 * <ul>
 * <li>specifying those elements which should be stripped
 * <li>specifying those elements whose tags and content should be
 *     completely removed from the event stream; and
   <li>specfiying those attributes that should be removed from an element
 * </ul>
 * <p>
 * The first option allows the application to specify which elements
 * appearing in the event stream should get their start and end tags removed.
 * The content, that means everything between both tags, are further
 * processed on to the next stage in the pipeline.
 * <p>
 * The second option allows the application to specify which elements
 * should be completely removed from the event stream. When an element
 * appears that is to be removed, the element's start and end tag as
 * well as all of that element's content is removed from the event
 * stream.
 * <p>
 * The third option allows the application to remove attributes from elements.
 * <p>
 * A common use of this filter would be to filter insecure and dangerous HTML
 * tags or element attributes, such as script, object or onLoad, from the
 * stream.
 * <pre>
 *  ElementRemover remover = new ElementRemover();
 *  remover.removeElement("object");
 *  remover.removeElement("script", null);
 *  remover.removeAttribute("onLoad");
 * </pre>
 * <p>
 * However, this would still allow safe content of other
 * elements to pass through.
 * <p>
 * <strong>Note:</strong>
 * All text and accepted element children of a stripped element is
 * retained. To completely remove an element's content, use the
 * <code>removeElement</code> method.
 * <p>
 * <strong>Note:</strong>
 * Care should be taken when using this filter because the output
 * may not be a well-balanced tree. Specifically, if the application
 * removes the &lt;HTML&gt; element (with or without retaining its
 * children), the resulting document event stream will no longer be
 * well-formed.
 *
 * @author Andy Clark, Janek Schwarz
 *
 * @version $Id: ElementRemover.java,v 1.2 2003/08/05 19:04:07 schwarz Exp $
 */
public class ElementRemover
  extends DefaultFilter
{
private static final Predicate DEFAULT_REMOVE_ATTRIBUTE_PREDICATE =
  new Predicate()
  {
    public boolean evaluate(Object in)
    {
      return true;
    }
  };

private Category log = Category.getInstance(ElementRemover.class);

//
// Data
//

// information

/** Stripped elements. */
protected Set fstrippedElements = new HashSet();

/** Removed elements. */
protected Set fRemovedElements = new HashSet();

/** Removed attributes. */
protected Map fRemovedAttributes = new HashMap();

// state

/** The element depth. */
protected int fElementDepth;

/** The element depth at element removal. */
protected int fRemovalElementDepth;

//
// Public methods
//

/**
 * Specifies that the given element should be stripped, that
 * means, the surrounding tags are removed while the content
 * contained within the element will be further processed.
 *
 * @param element The element to strip.
 *
 * see #removeElement
 */
public void stripElement(String element)
{
  fstrippedElements.add(element.toLowerCase());
} // stripElement(String,String[])

/**
 * Specifies that the given element should be completely removed. If an
 * element is encountered during processing that is on the remove list,
 * the element's start and end tags as well as all of content contained
 * within the element will be removed from the processing stream.
 *
 * @param element The element to completely remove.
 */
public void removeElement(String element)
{
  fRemovedElements.add(element.toLowerCase());
} // removeElement(String)

/**
 * Specifies that the given attribute should be removed from all
 * elements.
 *
 * @param attribute The attribute to completely remove.
 */
public void removeAttribute(String attribute)
{
  removeAttribute(attribute, DEFAULT_REMOVE_ATTRIBUTE_PREDICATE);
} // removeAttribute(String)

/**
 * Specifies that the given attribute should be removed from an element if the
 * predicate yields true. When an element has the given attribute, the
 * predicate is evaluated. If it returns true, the attribute is removed.
 *
 * @param attribute The attribute to completely remove
 * @param predicate Predicate
 */
public void removeAttribute(String attribute, Predicate predicate)
{
  fRemovedAttributes.put(attribute.toLowerCase(), predicate);
}

//
// XMLDocumentHandler methods
//

// since Xerces-J 2.2.0

/** Start document. */
public void startDocument(XMLLocator locator,
                          String encoding,
                          NamespaceContext nscontext,
                          Augmentations augs)
  throws XNIException
{
  if(log.isDebugEnabled())
    log.debug("startDocument -- start");

  fElementDepth = 0;
  fRemovalElementDepth = Integer.MAX_VALUE;
  super.startDocument(locator, encoding, nscontext, augs);

  if(log.isDebugEnabled())
    log.debug("startDocument -- end");
} // startDocument(XMLLocator,String,NamespaceContext,Augmentations)

// old methods

/** Start document. */
public void startDocument(XMLLocator locator,
                          String encoding,
                          Augmentations augs)
  throws XNIException
{
  startDocument(locator, encoding, null, augs);
} // startDocument(XMLLocator,String,Augmentations)

/** Start prefix mapping. */
public void startPrefixMapping(String prefix,
                               String uri,
                               Augmentations augs)
  throws XNIException
{
  if (fElementDepth <= fRemovalElementDepth)
    {
      super.startPrefixMapping(prefix, uri, augs);
    }
} // startPrefixMapping(String,String,Augmentations)

/** Start element. */
public void startElement(QName element,
                         XMLAttributes attributes,
                         Augmentations augs)
  throws XNIException
{
  if(log.isDebugEnabled())
    {
      log.debug("startElement -- start: ");
      log.debug("  element: " + element.rawname);
      log.debug("  fElementDepth: " + fElementDepth);
      log.debug("  fRemovalElementDepth: " + fRemovalElementDepth);
    }

  if (fElementDepth <= fRemovalElementDepth &&
      handleOpenTag(element, attributes))
    {
      super.startElement(element, attributes, augs);
    }

  fElementDepth++;

  if(log.isDebugEnabled())
    {
      log.debug("  increment fElementDepth: " + fElementDepth);
      log.debug("startElement -- end");
    }
} // startElement(QName,XMLAttributes,Augmentations)

/** Empty element. */
public void emptyElement(QName element,
                         XMLAttributes attributes,
                         Augmentations augs)
  throws XNIException
{
  if(log.isDebugEnabled())
    {
      log.debug("emptyElement -- start: ");
      log.debug("  element: " + element.rawname);
      log.debug("  fElementDepth: " + fElementDepth);
      log.debug("  fRemovalElementDepth: " + fRemovalElementDepth);
    }

  if (fElementDepth <= fRemovalElementDepth &&
      handleOpenTag(element, attributes))
    {
      super.emptyElement(element, attributes, augs);
    }

  if(log.isDebugEnabled())
    log.debug("emptyElement -- end: ");
} // emptyElement(QName,XMLAttributes,Augmentations)

/** Comment. */
public void comment(XMLString text, Augmentations augs)
  throws XNIException
{
  if(log.isDebugEnabled())
    {
      log.debug("comment -- start: ");
      log.debug("  text: " + text);
      log.debug("  fElementDepth: " + fElementDepth);
      log.debug("  fRemovalElementDepth: " + fRemovalElementDepth);
    }

  if (fElementDepth <= fRemovalElementDepth)
    {
      super.comment(text, augs);
    }

  if(log.isDebugEnabled())
    log.debug("comment -- end");
} // comment(XMLString,Augmentations)

/** Processing instruction. */
public void processingInstruction(String target,
                                  XMLString data,
                                  Augmentations augs)
  throws XNIException
{
  if (fElementDepth <= fRemovalElementDepth)
    {
      super.processingInstruction(target, data, augs);
    }
} // processingInstruction(String,XMLString,Augmentations)

/** Characters. */
public void characters(XMLString text,
                       Augmentations augs)
  throws XNIException
{
  if(log.isDebugEnabled())
    {
      log.debug("characters -- start: ");
      log.debug("  text: " + text);
      log.debug("  fElementDepth: " + fElementDepth);
      log.debug("  fRemovalElementDepth: " + fRemovalElementDepth);
    }

  if (fElementDepth <= fRemovalElementDepth)
    {
      super.characters(text, augs);
    }

  if(log.isDebugEnabled())
    log.debug("characters -- end");
} // characters(XMLString,Augmentations)

/** Ignorable whitespace. */
public void ignorableWhitespace(XMLString text,
                                Augmentations augs)
  throws XNIException
{
  if(log.isDebugEnabled())
    {
      log.debug("ignorableWhitespace -- start: ");
      log.debug("  text: " + text);
      log.debug("  fElementDepth: " + fElementDepth);
      log.debug("  fRemovalElementDepth: " + fRemovalElementDepth);
    }

  if (fElementDepth <= fRemovalElementDepth)
    {
      super.ignorableWhitespace(text, augs);
    }

  if(log.isDebugEnabled())
    log.debug("ignorableWhitespace -- end");
} // ignorableWhitespace(XMLString,Augmentations)

/** Start general entity. */
public void startGeneralEntity(String name,
                               XMLResourceIdentifier id,
                               String encoding,
                               Augmentations augs)
  throws XNIException
{
  if (fElementDepth <= fRemovalElementDepth)
    {
      super.startGeneralEntity(name, id, encoding, augs);
    }
} // startGeneralEntity(String,XMLResourceIdentifier,String,Augmentations)

/** Text declaration. */
public void textDecl(String version,
                     String encoding,
                     Augmentations augs)
  throws XNIException
{
  if (fElementDepth <= fRemovalElementDepth)
    {
      super.textDecl(version, encoding, augs);
    }
} // textDecl(String,String,Augmentations)

/** End general entity. */
public void endGeneralEntity(String name,
                             Augmentations augs)
  throws XNIException
{
  if (fElementDepth <= fRemovalElementDepth)
    {
      super.endGeneralEntity(name, augs);
    }
} // endGeneralEntity(String,Augmentations)

/** Start CDATA section. */
public void startCDATA(Augmentations augs)
  throws XNIException
{
  if (fElementDepth <= fRemovalElementDepth)
    {
      super.startCDATA(augs);
    }
} // startCDATA(Augmentations)

/** End CDATA section. */
public void endCDATA(Augmentations augs)
  throws XNIException
{
  if (fElementDepth <= fRemovalElementDepth)
    {
      super.endCDATA(augs);
    }
} // endCDATA(Augmentations)

/** End element. */
public void endElement(QName element,
                       Augmentations augs)
  throws XNIException
{
  if(log.isDebugEnabled())
    {
      log.debug("endElement -- start: ");
      log.debug("  element: " + element);
      log.debug("  fElementDepth: " + fElementDepth);
      log.debug("  fRemovalElementDepth: " + fRemovalElementDepth);
    }

  if (fElementDepth <= fRemovalElementDepth &&
      elementAccepted(element.rawname))
    {
      super.endElement(element, augs);
    }

  fElementDepth--;

  if (fElementDepth == fRemovalElementDepth)
    {
      fRemovalElementDepth = Integer.MAX_VALUE;
    }

  if(log.isDebugEnabled())
    {
      log.debug("  fElementDepth decremented: " + fElementDepth);
      log.debug("endElement -- start: ");
    }
} // endElement(QName,Augmentations)

/** End prefix mapping. */
public void endPrefixMapping(String prefix,
                             Augmentations augs)
  throws XNIException
{
  if (fElementDepth <= fRemovalElementDepth)
    {
      super.endPrefixMapping(prefix, augs);
    }
} // endPrefixMapping(String,Augmentations)

//
// Protected methods
//

/**
 * Returns true if the specified element is accept, that is, it
 * is not removed or stripped.
 */
protected boolean elementAccepted(String element)
{
  if(log.isDebugEnabled())
    {
      log.debug("elementAccepted -- start");
      log.debug("  element: " + element);
    }

  boolean accepted = !(elementStripped(element) || elementRemoved(element));

  if(log.isDebugEnabled())
    {
      log.debug("  accepted: " + accepted);
      log.debug("elementAccepted -- end");
    }

  return accepted;
}

/** Returns true if the specified element is stripped. */
protected boolean elementStripped(String element)
{
  if(log.isDebugEnabled())
    {
      log.debug("elementStripped -- start");
      log.debug("  element: " + element);
    }

  Object key = element.toLowerCase();
  boolean stripped = fstrippedElements.contains(key);

  if(log.isDebugEnabled())
    {
      log.debug("  stripped: " + stripped);
      log.debug("elementStripped -- end");
    }

  return stripped;
} // elementStripped(String):boolean

/** Returns true if the specified element should be removed. */
protected boolean elementRemoved(String element)
{
  if(log.isDebugEnabled())
    {
      log.debug("elementRemoved -- start");
      log.debug("  element: " + element);
    }

  Object key = element.toLowerCase();

  boolean removed = fRemovedElements.contains(key);

  if(log.isDebugEnabled())
    {
      log.debug("  removed: " + removed);
      log.debug("elementRemoved -- end");
    }

  return removed;
} // elementRemoved(String):boolean

/** Handles an open tag. */
protected boolean handleOpenTag(QName element,
                                XMLAttributes attributes)
{
  if(log.isDebugEnabled())
    {
      log.debug("handleOpenTag -- start");
      log.debug("  element: " + element.rawname);
    }

  boolean ret;

  if(elementAccepted(element.rawname))
    {
      Iterator removedAttributesIter =
        fRemovedAttributes.entrySet().iterator();

      int attributeCount = attributes.getLength();
    LOOP:
      for (int i = 0; i < attributeCount; i++)
        {
          String aname = attributes.getQName(i).toLowerCase();
          while(removedAttributesIter.hasNext())
            {
              Map.Entry attributeEntry =
                (Map.Entry) removedAttributesIter.next();
              String removedAttributeName =
                (String) attributeEntry.getKey();
              String removedAttributeValue =
                (String) attributes.getValue(removedAttributeName);
              Predicate removalPredicate =
                (Predicate) attributeEntry.getValue();
              if(removedAttributeName.equals(aname) &&
                 removalPredicate.evaluate(removedAttributeValue))
                {
                  attributes.removeAttributeAt(i--);
                  attributeCount--;

                  continue LOOP;
                }
            }
        }

      ret = true;
    }
  else if(elementRemoved(element.rawname))
    {
      if(log.isDebugEnabled())
        log.debug("  in else if(elementRemoved(...))");

      fRemovalElementDepth = fElementDepth;

      if(log.isDebugEnabled())
        log.debug("  set fRemovalElementDepth: " + fRemovalElementDepth);

      ret = false;
    }
  else if(elementStripped(element.rawname))
    {
      if(log.isDebugEnabled())
        log.debug("  in else if(elementStripped(...))");

      ret =  false;
    }
  else
    {
      if(log.isDebugEnabled())
        log.debug("  in else. should not happen");

      ret =  true;
    }

  if(log.isDebugEnabled())
    {
      log.debug("  ret: " + ret);
      log.debug("handleOpenTag -- end");
    }

  return ret;
} // handleOpenTag(QName,XMLAttributes):boolean
} // class ElementRemover
