package org.jaggregator.rss.common;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.net.URL;
import java.util.Date;
import org.apache.ecs.factory.DOMFactory;
import org.apache.ecs.xml.XMLDocument;
import org.custommonkey.xmlunit.DetailedDiff;
import org.custommonkey.xmlunit.Diff;
import org.jaggregator.rss.Channel;
import org.jaggregator.rss.Parser;
import org.jaggregator.rss.Subscription;
import org.jaggregator.rss.rss_0_91.Channel_0_91;
import org.jaggregator.rss.rss_0_91.Parser_0_91;
import org.jaggregator.rss.util.FileUtils;
import org.jaggregator.util.XMLTestCase;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;

/**
 * Unit Test for class TestSubscriptionExporterImpl.java
 *
 * @author <a href="mailto:jsz@jaggregator.org">Janek Schwarz</a>
 * @version $Revision: 1.7 $
 */
public class TestSubscriptionExporterImpl
  extends XMLTestCase 
{
private Subscription subscription;

public TestSubscriptionExporterImpl (String name)
{
  super(name);
}

protected void setUp()
  throws Exception
{
  Parser parser = new Parser_0_91();
  
  Channel channel = (Channel_0_91) parser.parse(
    createInputSourceForFile(
      "src/test/org/jaggregator/rss/rss_0_91/rss-example-0.91.2.xml"));

  subscription = new SubscriptionImpl();
  Date subscriptionDate = new Date(0);
  subscription.setSubscriptionDate(subscriptionDate);

  Date lastRetrievedDate = new Date(1000);
  subscription.setLastRetrievedDate(lastRetrievedDate);
  subscription.setLastModifiedString("fake-last-modified");
  subscription.setEtagString("fake-etag");
  
  subscription.setUrl(
    new URL("http://www.jaggregator.org/testbed/subscriptiontestchannel.xml"));
  
  subscription.setChannel(channel);
}

public void testConstructorNonExistingParentDirectory()
{
  SubscriptionExporterImpl exporter = 
    new SubscriptionExporterImpl(new File("target/subscription-tests"));

  //all we want is that the constructor does not throw an exception
}

public void testConstructorExistingParentDirectory()
{
  SubscriptionExporterImpl exporter = 
    new SubscriptionExporterImpl(new File("target"));

  //all we want is that the constructor does not throw an exception
}

public void testConstructorExistingParentDirectoryIsFile()
{
  try
    {
      SubscriptionExporterImpl exporter = 
        new SubscriptionExporterImpl(createFile("build.xml"));
      fail("Expected IllegalArgumentException");
    }
  catch(IllegalArgumentException ex)
    {
      //all we want is that the constructor throws an exception
    }
}

public void testCreateSubscriptionXMLDocument()
  throws Exception
{
  SubscriptionExporterImpl exporter = 
    new SubscriptionExporterImpl(
      createFile("target/subscription-tests"));
  
  XMLDocument subscriptionXMLDocument =
    exporter.createSubscriptionXMLDocument(subscription);

  ByteArrayOutputStream ostream = new ByteArrayOutputStream();
  subscriptionXMLDocument.output(ostream);
  Document actualValue = createDocument(
    new ByteArrayInputStream(ostream.toByteArray()));
  Document expectedValue = createDocument(
    createInputStreamForFile(
      "src/test/org/jaggregator/rss/common/subscriptiontest.xml"));
  
  DetailedDiff detailedDiff = new DetailedDiff(new Diff(expectedValue,
                                                        actualValue));
  assertXMLEqual("", detailedDiff, true);
}

public void testCreateSubscriptionXMLDocumentWOChannelData()
  throws Exception
{
  subscription = new SubscriptionImpl();
  Date subscriptionDate = new Date(0);
  subscription.setSubscriptionDate(subscriptionDate);

  Date lastRetrievedDate = new Date(1000);
  subscription.setLastRetrievedDate(lastRetrievedDate);

  subscription.setUrl(
    new URL("http://www.jaggregator.org/testbed/subscriptiontestchannel.xml"));

  SubscriptionExporterImpl exporter = 
    new SubscriptionExporterImpl(new File("target/subscription-tests"));
  XMLDocument subscriptionXMLDocument =
    exporter.createSubscriptionXMLDocument(subscription);

  ByteArrayOutputStream ostream = new ByteArrayOutputStream();
  subscriptionXMLDocument.output(ostream);
  Document actualValue = createDocument(
    new ByteArrayInputStream(ostream.toByteArray()));
  Document expectedValue = createDocument(
    createInputStreamForFile(
      "src/test/org/jaggregator/rss/common/subscriptiontest2.xml"));
  
  DetailedDiff detailedDiff = new DetailedDiff(new Diff(expectedValue,
                                                        actualValue));
  assertXMLEqual("", detailedDiff, true);
}

public void testExport()
  throws Exception
{
  File allSubscriptionsRoot = new File("target/subscription-tests");
  
  SubscriptionExporterImpl exporter = 
    new SubscriptionExporterImpl(allSubscriptionsRoot);

  exporter.export(subscription);
  
  assertTrue(allSubscriptionsRoot.exists());

  File subscriptionDir = new File(allSubscriptionsRoot,
                                  FileUtils.uriToFileName(
                                    subscription.getUrl().toString()));
  assertTrue(subscriptionDir.exists());

  File subscriptionFile =
    new File(subscriptionDir,
             SubscriptionExporterImpl.SUBSCRIPTION_FILE_NAME);
  assertTrue(subscriptionFile.exists());

  File channelFile =
    new File(subscriptionDir,
             SubscriptionExporterImpl.CHANNEL_FILE_NAME);
  assertTrue(channelFile.exists());

  channelFile.delete();
  subscriptionFile.delete();
  subscriptionDir.delete();
  allSubscriptionsRoot.delete();
}

protected void tearDown()
{
  assertFalse(new File("target/subscription-tests").exists());
}
}
