package org.jaggregator.rss.common;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.URL;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Date;
import org.jaggregator.TestCase;
import org.jaggregator.rss.Subscription;
import org.jaggregator.rss.Version;
import org.xml.sax.InputSource;

/**
 * Unit Test for class TestSubscriptionParserImpl.java
 *
 * @author <a href="mailto:jsz@jaggregator.org">Janek Schwarz</a>
 * @version $Revision: 1.7 $
 */
public class TestSubscriptionParserImpl
  extends TestCase 
{
private File subscriptionsRoot = 
  createFile("src/test/org/jaggregator/rss/common/subscriptionsroot");
private String channelUriString =
    "http://www.jaggregator.org/testbed/subscriptiontestchannel.xml";
private URL channelUri;

public TestSubscriptionParserImpl(String name)
{
  super(name);
}

protected void setUp()
  throws Exception
{
  channelUri = new URL(channelUriString);
}

public void testConstructorWithExistingRoot()
  throws Exception
{
  SubscriptionParserImpl parser = 
    new SubscriptionParserImpl(subscriptionsRoot);

  assertNotNull(parser);
}

public void testConstructorWithNotExistingRoot()
  throws Exception
{
  try
    {
      SubscriptionParserImpl parser = 
        new SubscriptionParserImpl(new File("abc"));
      
      fail("Expected FileNotFoundException");
    }
  catch(FileNotFoundException ex)
    {
      // test passed
    }
}

public void testConstructorWithFile()
  throws Exception
{
  try
    {
      SubscriptionParserImpl parser = 
        new SubscriptionParserImpl(new File("build.xml"));
      
      fail("Expected IOException");
    }
  catch(IOException ex)
    {
      // test passed
    }
}

public void testCreateSubscriptionFileIsourceWithExistingSubscription()
  throws Exception
{
  SubscriptionParserImpl parser = 
    new SubscriptionParserImpl(subscriptionsRoot);
  
  InputSource actualValue = parser.createSubscriptionFileIsource(
    channelUri);

  assertNotNull(actualValue);
}

public void testCreateSubscriptionFileIsourceWithNonExistingSubscription()
  throws Exception
{
  SubscriptionParserImpl parser = 
    new SubscriptionParserImpl(subscriptionsRoot);
  
  try
    {
      InputSource actualValue = parser.createSubscriptionFileIsource(
        new URL("http://does.not.exist"));

      fail("Expected FileNotFoundException");
    }
  catch(FileNotFoundException ex)
    {
      // test passed
    }
}

public void testParse()
  throws Exception
{
  SubscriptionParserImpl parser = 
    new SubscriptionParserImpl(subscriptionsRoot);
  
  Subscription subscription = parser.parse(channelUri);
  
  assertNotNull(subscription);
}

public void testUrl()
  throws Exception
{
  SubscriptionParserImpl parser = 
    new SubscriptionParserImpl(subscriptionsRoot);
  
  Subscription subscription = parser.parse(channelUri);
  
  URL actualValue = subscription.getUrl();
  URL expectedValue = 
    new URL("http://www.jaggregator.org/testbed/subscriptiontestchannel.xml");
  
  assertEquals(expectedValue, actualValue);
}

public void testSubscriptionDate()
  throws Exception
{
  SubscriptionParserImpl parser = 
    new SubscriptionParserImpl(subscriptionsRoot);
  
  Subscription subscription = parser.parse(channelUri);
  
  Date actualValue = subscription.getSubscriptionDate();
  Date expectedValue = new Date(0);
  
  assertEquals(expectedValue, actualValue);
}

public void testLastRetrievedDate()
  throws Exception
{
  SubscriptionParserImpl parser = 
    new SubscriptionParserImpl(subscriptionsRoot);
  
  Subscription subscription = parser.parse(channelUri);
  
  Date actualValue = subscription.getLastRetrievedDate();
  Date expectedValue = new Date(1000);
  
  assertEquals(expectedValue, actualValue);
}

public void testChannelLevelChanges()
  throws Exception
{
  SubscriptionParserImpl parser = 
    new SubscriptionParserImpl(subscriptionsRoot);
  
  Subscription subscription = parser.parse(channelUri);
  
  boolean actualValue = subscription.hasChannelLevelChanges();
  boolean expectedValue = true;
  
  assertEquals(expectedValue, actualValue);
}

public void testChannelInfos()
  throws Exception
{
  SubscriptionParserImpl parser = 
    new SubscriptionParserImpl(subscriptionsRoot);
  
  SubscriptionImpl subscription = (SubscriptionImpl) parser.parse(channelUri);

  assertNotNull(parser.channelInfos);
}

public void testChannelVersion()
  throws Exception
{
  SubscriptionParserImpl parser = 
    new SubscriptionParserImpl(subscriptionsRoot);
  
  SubscriptionImpl subscription = (SubscriptionImpl) parser.parse(channelUri);

  Version actualValue = parser.channelInfos.version;
  Version expectedValue = Version.VERSION_0_91;
  
  assertEquals(expectedValue, actualValue);
}

public void testHashDateMap()
  throws Exception
{
  SubscriptionParserImpl parser = 
    new SubscriptionParserImpl(subscriptionsRoot);
  
  SubscriptionImpl subscription = (SubscriptionImpl) parser.parse(channelUri);

  int actualValue = parser.channelInfos.itemDateMap.size();
  int expectedValue = 4;
  
  assertEquals(expectedValue, actualValue);
}

public void testChannel()
  throws Exception
{
  SubscriptionParserImpl parser = 
    new SubscriptionParserImpl(subscriptionsRoot);
  
  SubscriptionImpl subscription =
    (SubscriptionImpl) parser.parse(channelUri);

  assertNotNull(subscription.getChannel());
}

public void testChannelDateItemMap()
  throws Exception
{
  SubscriptionParserImpl parser = 
    new SubscriptionParserImpl(subscriptionsRoot);
  
  SubscriptionImpl subscription = (SubscriptionImpl) parser.parse(channelUri);
  
  int actualValue = subscription.dateItemMap.size();
  int expectedValue = 1;
  assertEquals(expectedValue, actualValue);
  
  int actualValueItem = 
    ((ArrayList) subscription.dateItemMap.get(new Date(1000))).size();
  int expectedValueItem = 4;
  assertEquals(expectedValueItem, actualValueItem);
}
}
